import socket

verbose = False
import select
import errno
import sys


def debug_print(*args):
    if verbose:
        print 'DEBUG_PRINT: ',
        for arg in args:
            print arg,
        print

class Update:
    payload = ''
    dest = []
    version = 0
    src = ''

class Message:
    update_msg_list = {}
    messenger_type = ''
    network = []

    def construct_update(self, dest, name, payload):
        assert type(dest) == list and type(name) == str and type(payload) == str
        #if not in list, construct one
        if not name in self.update_msg_list.keys():
            self.update_msg_list[name] = Update()
        update = self.update_msg_list[name]
        if len(dest) == 0:
            return ''
        #set update info
        update.payload = payload
        update.dest = dest
        update.version +=1
        update.src = self.messenger_type
        dstr = ''
        for d in dest:
            dstr += ';'+d
        return 'U/{}/{}/{}/{}/{}'.format(self.messenger_type,dstr,name,update.version,payload)

    def reconstruct_update(self, name):
        assert name in self.update_msg_list.keys()
        update = self.update_msg_list[name]
        #set update info
        dstr = ''
        for d in update.dest:
            dstr += ';'+d
        print update.src
        msg = 'U/{}/{}/{}/{}/{}'.format(update.src,dstr,name,update.version,update.payload)
        print msg
        return msg
    
    def update_msg(self, msg):
        msg_split = msg.strip("\n").split('/')
        #debug_print(msg_split)
        i = 0
        while(msg_split[i] == ''):
            i += 1
        src = msg_split[i]
        dest = msg_split[i+1].split(';')
        if '' in dest:
            dest.remove('')
        #disgard msg if the message is not for you
        if self.messenger_type != 'msg_server' and not self.messenger_type in dest:
            return None
        #get update name, if not in list, construct one
        name = msg_split[i+2]
        if not name in self.update_msg_list.keys():
            self.update_msg_list[name] = Update()
        update = self.update_msg_list[name]
        #get version, payload
        version = (int)(msg_split[i+3])
        payload = msg_split[i+4]
        #disgard msg if it is not a newer one
        if version <= update.version:
            return None
        #set update info
        update.dest = dest
        update.payload = payload
        update.version = version
        return name

    def _resolve_msg(self, s, msg):
        pass

    def recv(self, s):
        buf = ''
        while(True):            
            try:
                data = s.recv(1024)
            except socket.error, e:
                err = e.args[0]
                if err == errno.EAGAIN or err == errno.EWOULDBLOCK:
                    debug_print ('message recv: No data available')
                    break
                else:
                    # Something else happened, handle error, exit, etc.
                    print e
                    sys.exit(1)
            else:
                if data:
                    #debug_print('received message', data)
                    buf += data
                else:
                    print "message recv fails"
                    self.handle_lose_connection(s)
                    break
        line_list = buf.split('\n')
        for line in line_list:
            self._resolve_msg(s, line)

    def update_print(self, s):
        if s in self.update_msg_list.keys():
            x = self.update_msg_list[s].payload
            print x

    def get_update(self, s):
        if s in self.update_msg_list.keys():
            x = self.update_msg_list[s].payload
            return x
        else:
            return None

    def handle_lose_connection(self, s):
        print "lose_connection"
        if not s is None:
            s.close()


class Client(Message):
    connected = False
    initiated = False
    messenger_type = ''
    server = None
    network = []
    closed = False


    def __init__(self, messenger_type):
        self.messenger_type = messenger_type
        


    def try_connect(self, port, host):
        try:
            self.server.connect((host, port))
            debug_print("client try connect: connected!")
        except:
            print "message server not found!"
            self.close()
            

    def init_connections(self, port, host):
        sock = socket.socket()
        self.server = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.try_connect(port, host)
        self.connected = True
        self.server.setblocking(0)

    def send(self, msg):
        try:
            self.server.send(msg+'\n')
        except:
            print "not able to send"
            self.handle_lose_connection(self.server)

    def poll_msg(self, reads, writes, excepts):
        assert type(reads) == list and type(writes) == list and type(excepts) == list
        readable, writable, exceptional = select.select(reads, writes, excepts, 0)
        for s in readable:
            self.recv(self.server)


    def update_network(self):
        if 'network' in self.update_msg_list.keys():
            self.network = self.update_msg_list['network'].payload.split(';')

# =========== helper function =============================
    def close(self):
        print "closing"
        self.server.close()
        self.closed = True

    def _resolve_update_msg(self, name, payload):
        pass

    def close_request(self):
        print "sending close request"
        self.send('X:'+self.messenger_type)
        #self.close()

    def _resolve_msg(self, s, msg):
        if msg == '':
            return
        Message._resolve_msg(self, self.server, msg)
        if msg[0] == 'T':
            self.send('T/' + self.messenger_type + '/')
            debug_print('client resolve msg: ', "sending type to host")
        if msg[0] == 'I':
            self.initiated = True
            self.send('I/' + self.messenger_type + '/')
            debug_print('client resolve msg: ', 'system initiated!')
        if msg[0] == 'C':
            debug_print(msg)
        if msg[0] == 'U':
            debug_print ('client resolve msg: ', 'update', msg)
            name = self.update_msg(msg[1:])
            debug_print(name)
            if name in self.update_msg_list.keys():
                debug_print ('client resolve msg: ', self.update_msg_list[name].payload)
                self._resolve_update_msg(name, self.update_msg_list[name].payload)
        if msg[0] == 'X': #shutdown system
            print "receive shut down"
            debug_print ('closing socket... shutting down system', self.messenger_type)
            self.close()

    def handle_lose_connection(self, s):
        self.close()
        #Message.handle_lose_connection(self, s)
        #if not self.closed:
            #self.connected = False
            #self.initiated = False
            #self.network = []
            #self.port = 12345
            #self.host = socket.gethostname()
            #self.init_connections(self.port, self.host)

