import socket
from client import *
from time import sleep,ctime,time
import subprocess

class Signal():
    connected = False
    initiated = False
    time = 0

'''A security center does the following:
    1. actively logs the events into a local file
    2. logging level is based on security mode
    3. takes picture
    4. scans the apartment based on security mode
    5. responds to clicks based on security mode'''

class SecurityCenter(Client):
    mode = 0 # 0: manual, 1: easy secure, 2: front door focused, 3: heavily logged
    filename = ""

    def __init__(self, type, mode = 1, filename = "activity.log"):
        Client.__init__(self, type)
        self.mode = mode
        self.filename = filename
        
    def resolve_msg(self, msg):
        if msg == '':
            return
        if self.mode == 3:
            if msg[0] == 'I':
                self.log_msg("Initialized", self.messenger_type)
            if msg[0] == 'X':
                self.log_msg("Closing", self.messenger_type)
        Client.resolve_msg(self, msg)

    def log_msg(self, event, detail):
        cmd = "echo '" + ctime(time()) + '$' + event + '$' + detail + "' >> " + self.filename
        print cmd
        subprocess.Popen(cmd, shell=True)

    def clr_log(self):
        subprocess.Popen("echo '' > " + self.filename, shell=True)

    def send_status_update(self, name, payload):
        if len(self.network) != 0:
            s = self.construct_update(self.network, name+'_status', str(payload))
            self.send(s)

    def _resolve_update_msg(self, name, payload):
        debug_print("resolve update msg: ", name, payload)
        if name == "set_mode":
            self.mode = int(payload)
            Signal.time = time()
            self.log_msg("set_mode", payload)
            self.send_status_update(name, payload)
            return
        if self.mode == 2:
            if name == "front_door_btn":
                self.send(self.construct_update(self.network, "servo_setmode", "0"))
            if name == "balcony_btn":
                self.send(self.construct_update(self.network, "servo_setmode", "1"))
            return
        if name == "get_mode":
            self.send_status_update(name, str(self.mode))
        if name == "clr_log":
            self.clr_log()
            self.send_status_update(name, payload)
            return
        if self.mode == 1 or self.mode == 3:
            # move when triggered
            if name == "front_door_btn":
                s = self.construct_update(self.network, "target_front_door", "0")
                self.send(s)
                self.log_msg(name, payload)
                self.log_msg("target_front_door", "action detected")
            elif name == "balcony_btn":
                s = self.construct_update(self.network, "target_balcony", "0")
                self.send(s)
                self.log_msg(name, payload)
                self.log_msg("target_balcony", "action detected")
            else:
                if self.mode == 3:
                    self.log_msg(name, payload)

    def close(self):
        Client.close(self)


port = 12345
host = socket.gethostname()

demo_events = {
    3.00: {"bedroom_light_1":"true", "bedroom_light_2": "true"},
    6.00: {"dining_room_light_1":"true"},
    9.00: {"dining_room_light_1" : "true"},
    12.00: {"living_room_light_1":"true"},
    15.00: {"living_room_light_1" : "false", "bedroom_light_1":"false", "bedroom_light_2": "false"}
}
demo_events_time = demo_events.keys()
demo_events_time.sort()


ctrl = SecurityCenter('security_center', 1, "activity.log")
ctrl.init_connections(port, host)
Signal.time = time()
Signal.ind = 0

count = 0
while(not ctrl.closed):
    sleep(0.1)
    ctrl.poll_msg([ctrl.server],[],[])
    ctrl.update_network()
    if ctrl.mode == 2:
        if time()-Signal.time >= demo_events_time[Signal.ind]:
            Signal.ind += 1
            if Signal.ind >= len(demo_events_time):
                Signal.ind = 0
            for key in demo_events[demo_events_time[Signal.ind]]:
                s = ctrl.construct_update(ctrl.network, key, demo_events[demo_events_time[Signal.ind]][key])
                ctrl.poll_msg([ctrl.server],[],[])
                print s
                ctrl.send(s)
            if Signal.ind == 0:
                Signal.time = time()
