import socket
from client import *
from time import sleep
import pigpio

class Servo():
    runnint = False
    current_angle = 0

    def __init__(self, pi, pin, freq = 200, dc_min = 110000, dc_max = 390000, dc_start = 250000):
        self.p = pi
        self.pin = pin
        self.freq = freq
        self.dc_min = dc_min
        self.dc_max = dc_max
        self.dc = dc_start
        self.current_angle = 50
        self.p.hardware_PWM(self.pin, self.freq, self.dc)

    def servo_set_angle(self,angle):
        # restrict angle to max/min value
        if angle > 100:
            angle = 100
        if angle < 0:
            angle = 0
        # save angle
        self.current_angle = angle
        # calculate dc
        dc = self.dc_min+(self.dc_max-self.dc_min)*angle/100
        self.dc = dc
        self.p.hardware_PWM(self.pin, self.freq, self.dc)
        debug_print('servo set angle: ', angle, dc)

    def servo_start(self):
        print "servo started"

    def servo_stop(self):
        self.p.write(self.pin, 0)

class Signal():
    connected = False
    initiated = False

class ServoCtrl(Client):
    servo_control_list = {'servo_azim' : None, 'servo_elev' : None}
    #mode = 0 #mode# 0: irresponsive to button 1: respond to button presses (at location) 2: periodical checking
    waypoints = {'front_door': [50, 50], 'balcony': [92, 54], 'gallery' : [20, 47]}

    def __init__(self, messenger_type, azim_pin=19, elev_pin=18, freq = 200, dc_min = 110000, dc_max = 390000, dc_start = 250000):
        Client.__init__(self, messenger_type)
        self.p = pigpio.pi()
        self.servo_control_list['servo_azim'] = Servo(self.p, azim_pin, freq, dc_min, dc_max, dc_start)
        self.servo_control_list['servo_elev'] = Servo(self.p, elev_pin, freq, dc_min, dc_max, dc_start)
        self.mode = 0

    def close(self):
        self.server.close()
        self.closed = True
        for i in self.servo_control_list:
            self.servo_control_list[i].servo_stop()    
    
    def resolve_msg(self, msg):
        Client.resolve_msg(self, msg)
        
    def send_status_update(self, name, payload):
        if len(self.network) != 0:
            s = self.construct_update(self.network, name+'_status', str(payload))
            self.send(s)

    def send_status_update(self, name, payload):
        if len(self.network) != 0:
            s = self.construct_update(self.network, name+'_status', str(payload))
            self.send(s)

    def set_position(self, azim, elev):
        self.servo_control_list['servo_azim'].servo_set_angle(azim)
        self.servo_control_list['servo_elev'].servo_set_angle(elev)

    def _resolve_update_msg(self, name, payload):
        # control servo
        if name in self.servo_control_list.keys() and payload.isdigit():
            payload = int(payload)
            self.servo_control_list[name].servo_set_angle(payload)
            self.send_status_update(name, str(self.servo_control_list[name].current_angle))
        elif (name == "target_front_door"):
            self.set_position(self.waypoints['front_door'][0],self.waypoints['front_door'][1])
        elif (name == "target_balcony"):
            self.set_position(self.waypoints['balcony'][0],self.waypoints['balcony'][1])
        elif (name == "target_gallery"):
            self.set_position(self.waypoints['gallery'][0],self.waypoints['gallery'][1])
        elif (name == "servo_setmode"):
            self.mode = int(payload)
        else:
            debug_print('unknown payload, ignoring ...')
            return
        self.send_status_update(name, payload)


port = 12345
host = socket.gethostname()
ctrl = ServoCtrl('servo_ctrl', 18, 19)
ctrl.init_connections(port, host)

angle = 0
direction = 0

count = 0
while(not ctrl.closed):
    sleep(0.2)
    ctrl.poll_msg([ctrl.server],[],[])
    debug_print ("servo_ctrl: ", 'ctrl.initiated: ', ctrl.initiated)
    ctrl.update_network()
    if ctrl.mode == 1:
        if direction == 0:
            angle += 1
        else:
            angle -= 1
        if angle>=100:
            angle = 100
            direction = 1
        if angle <= 0:
            angle = 0
            direction = 0
        ctrl.servo_control_list['servo_azim'].servo_set_angle(angle)
